@@ -4,7 +4,7 @@ import warnings
 from scipy.stats import entropy 
 from ..utility import *
 from ..custom import *
-#from ..custom.newmetric_child import *
+from ..custom.archive.newmetric_child import *
 import concurrent.futures
 
 class FairnessMetrics:
@@ -14,38 +14,38 @@ class FairnessMetrics:
     Class Attributes
     ----------
     map_fair_metric_to_group : dictionary
-        Maps the fairness metrics to its name, metric_group (classification, uplift, or regression) and type (parity or odds).
-        e.g. {'equal_opportunity': ('Equal Opportunity', 'classification', 'parity'), 'equal_odds': ('Equalized Odds', 'classification', 'parity')}
+        Maps the fairness metrics to its name, metric_group (classification, uplift, or regression), type (parity or odds), whether the metric is related to tradeoff and whether the metric can be a primary metric.
+        e.g. {'equal_opportunity': ('Equal Opportunity', 'classification', 'parity', True, True), 'equal_odds': ('Equalized Odds', 'classification', 'parity', True, True)}
     """
     map_fair_metric_to_group = {
-        'disparate_impact': ('Disparate Impact', 'classification', 'ratio', True),
-        'demographic_parity': ('Demographic Parity', 'classification', 'parity', True),
-        'equal_opportunity': ('Equal Opportunity', 'classification', 'parity', True),
-        'fpr_parity': ('False Positive Rate Parity', 'classification', 'parity', True),
-        'tnr_parity': ('True Negative Rate Parity', 'classification', 'parity', True),
-        'fnr_parity': ('False Negative Rate Parity', 'classification', 'parity', True),
-        'ppv_parity': ('Positive Predictive Parity', 'classification', 'parity', True),
-        'npv_parity': ('Negative Predictive Parity', 'classification', 'parity', True),
-        'fdr_parity': ('False Discovery Rate Parity', 'classification', 'parity', True),
-        'for_parity': ('False Omission Rate Parity', 'classification', 'parity', True),
-        'equal_odds': ('Equalized Odds', 'classification', 'parity', True),
-        'neg_equal_odds': ('Negative Equalized Odds', 'classification', 'parity', True),
-        'calibration_by_group': ('Calibration by Group', 'classification', 'parity', True),
-        'auc_parity': ('AUC Parity', 'classification', 'parity', False),
-        'log_loss_parity': ('Log-loss Parity', 'classification', 'parity', False),
-        'mi_independence': ('Mutual Information Independence', 'classification', 'information', False),
-        'mi_separation': ('Mutual Information Separation', 'classification', 'information', False),
-        'mi_sufficiency': ('Mutual Information Sufficiency', 'classification', 'information', False),
-        'rmse_parity': ('Root Mean Squared Error Parity', 'regression', 'parity', False),
-        'mape_parity': ('Mean Absolute Percentage Error Parity', 'regression', 'parity', False),
-        'rejected_harm': ('Harm from Rejection', 'uplift', 'parity', True),
-        'acquire_benefit': ('Benefit from Acquiring', 'uplift', 'parity', True)
+        'disparate_impact': ('Disparate Impact', 'classification', 'ratio', True, True),
+        'demographic_parity': ('Demographic Parity', 'classification', 'parity', True, True),
+        'equal_opportunity': ('Equal Opportunity', 'classification', 'parity', True, True),
+        'fpr_parity': ('False Positive Rate Parity', 'classification', 'parity', True, True),
+        'tnr_parity': ('True Negative Rate Parity', 'classification', 'parity', True, True),
+        'fnr_parity': ('False Negative Rate Parity', 'classification', 'parity', True, True),
+        'ppv_parity': ('Positive Predictive Parity', 'classification', 'parity', True, True),
+        'npv_parity': ('Negative Predictive Parity', 'classification', 'parity', True, True),
+        'fdr_parity': ('False Discovery Rate Parity', 'classification', 'parity', True, True),
+        'for_parity': ('False Omission Rate Parity', 'classification', 'parity', True, True),
+        'equal_odds': ('Equalized Odds', 'classification', 'parity', True, True),
+        'neg_equal_odds': ('Negative Equalized Odds', 'classification', 'parity', True, True),
+        'calibration_by_group': ('Calibration by Group', 'classification', 'parity', True, True),
+        'auc_parity': ('AUC Parity', 'classification', 'parity', False, True),
+        'log_loss_parity': ('Log-loss Parity', 'classification', 'parity', False, True),
+        'mi_independence': ('Mutual Information Independence', 'classification', 'information', False, False),
+        'mi_separation': ('Mutual Information Separation', 'classification', 'information', False, False),
+        'mi_sufficiency': ('Mutual Information Sufficiency', 'classification', 'information', False, False),
+        'rmse_parity': ('Root Mean Squared Error Parity', 'regression', 'parity', False, True),
+        'mape_parity': ('Mean Absolute Percentage Error Parity', 'regression', 'parity', False, True),
+        'wape_parity': ('Weighted Absolute Percentage Error Parity', 'regression', 'parity', False, True),
+        'rejected_harm': ('Harm from Rejection', 'uplift', 'parity', True, True),
+        'acquire_benefit': ('Benefit from Acquiring', 'uplift', 'parity', False, True)
     }
 
     for metric in newmetric.NewMetric.__subclasses__() :
         if metric.enable_flag ==True and metric.metric_type == "fair":
-            map_fair_metric_to_group[metric.metric_name] =  (metric.metric_definition, metric.metric_group, metric.metric_parity_ratio,True)
-
+            map_fair_metric_to_group[metric.metric_name] =  (metric.metric_definition, metric.metric_group, metric.metric_parity_ratio, False, False)
 
     def __init__(self, use_case_object):
         """
@@ -84,15 +84,15 @@ class FairnessMetrics:
         sample_weight : array of shape (n_samples,), default=None
                 Used to normalize y_true & y_pred.
 
-        p_var : list
+        p_var : list, default=None
                 List of protected variables used for fairness analysis.
 
-        fair_metric_name: string, default  = determined by fair_concern & model_type
+        fair_metric_name: string, default=None
                 Name of the primary fairness metric to be used for computations in the evaluate() and/or compile() functions
 
-        _use_case_metrics: dictionary of lists, default="None"
+        _use_case_metrics: dictionary of lists, default=None
                 Contains all the performance & fairness metrics for credit scoring.
-                {"fair ": ["fnr_parity", ...],"perf": ["balanced_accuracy, ..."]}
+                {"fair ": ["fnr_parity", ...], "perf": ["balanced_accuracy, ..."]}
                 Dynamically assigned during initialisation by using the _metric_group_map in Fairness/Performance Metrics class and the _model_type_to_metric above.
 
         """
@@ -108,7 +108,7 @@ class FairnessMetrics:
             'fdr_parity': self._compute_false_discovery_rate_parity,
             'for_parity': self._compute_false_omission_rate_parity,
             'equal_odds': self._compute_equalized_odds,
-            'neg_equal_odds': self._compute_equalized_odds,
+            'neg_equal_odds': self._compute_negative_equalized_odds,
             'calibration_by_group': self._compute_calibration_by_group,
             'auc_parity': self._compute_auc_parity,
             'log_loss_parity': self._compute_log_loss_parity,
@@ -117,15 +117,19 @@ class FairnessMetrics:
             'mi_sufficiency': self._compute_mi_sufficiency, 
             'rmse_parity': self._compute_rmse_parity,
             'mape_parity': self._compute_mape_parity,
+            'wape_parity': self._compute_wape_parity,
             'rejected_harm': self._compute_rejected_harm,
             'acquire_benefit': self._compute_benefit_from_acquiring
         }
 
-        #self.map_fair_metric_to_method = {'disparate_impact': self._compute_disparate_impact}
-        for metric in newmetric.NewMetric.__subclasses__() : # check if we can access NewMetric.__subclasses__() only one time
+        for metric in NewMetric.__subclasses__() :
             if metric.enable_flag ==True and metric.metric_type == "fair":
                 self.map_fair_metric_to_method[metric.metric_name] =  metric.compute
-
+                self.map_fair_metric_to_group[metric.metric_name] =  (metric.metric_definition, metric.metric_group, metric.metric_parity_ratio,True)
+                #print(self.map_perf_metric_to_method)
+                if metric.metric_name not in use_case_object._use_case_metrics["fair"]:
+                    use_case_object._use_case_metrics["fair"].append(metric.metric_name)
+                    
         self.result = {}
         self.y_true  = None
         self.y_prob  = None
@@ -138,7 +142,7 @@ class FairnessMetrics:
         self.use_case_object = use_case_object
 
 
-    def execute_all_fair(self, n_threads):
+    def execute_all_fair(self, n_threads, seed, eval_pbar):
 
         """
         Computes every fairness metric named inside the include_metrics list together with its associated confidence interval (dictionary), the privileged group metric value & the neutral position.
@@ -147,6 +151,12 @@ class FairnessMetrics:
         ----------
         use_case_object : object
                 A single initialized Fairness use case object (CreditScoring, CustomerMarketing, etc.)
+        
+        n_threads : int
+
+        seed : int
+                Used to initialize the random number generator.
+
 
         Returns
         ----------
@@ -178,13 +188,13 @@ class FairnessMetrics:
                 if j in self.map_fair_metric_to_method.keys():
                     self.result[i]["fair_metric_values"][j] = [] 
                   
-
+        eval_pbar.update(10)
         n = len(self.use_case_object.model_params[0].y_true)
         n_threads = check_multiprocessing(n_threads)
         if n_threads >= 1:
 
             indices = []
-            np.random.seed(123)
+            np.random.seed(seed)
             for ind in range(self.use_case_object.k-1):
                 indices.append(np.random.choice(n, n, replace=True))
 
@@ -195,7 +205,7 @@ class FairnessMetrics:
                     indexes[i].append(x)
 
             threads = []
-
+            worker_progress = 36/n_threads
             with concurrent.futures.ThreadPoolExecutor(max_workers = n_threads) as executor:
                 #iterate through protected variables to drop one by one as part of leave-on-out
                 for k in range(n_threads):
@@ -203,7 +213,7 @@ class FairnessMetrics:
                         metric_obj = self
                     else:
                         metric_obj = deepcopy(self)
-                    threads.append(executor.submit(FairnessMetrics._execute_all_fair_map, metric_obj=metric_obj, index =indexes[k]))
+                    threads.append(executor.submit(FairnessMetrics._execute_all_fair_map, metric_obj=metric_obj, index =indexes[k], eval_pbar=eval_pbar, worker_progress=worker_progress))
 
                 for thread in threads:
                     mp_result = thread.result()
@@ -215,7 +225,7 @@ class FairnessMetrics:
 
 
         idx = np.array(range(n))
-        FairnessMetrics._execute_all_fair_map(self, [idx])
+        FairnessMetrics._execute_all_fair_map(self, [idx], eval_pbar, 1)
 
         for i in self.p_var[0]:
             for j in self._use_case_metrics['fair']:
@@ -224,11 +234,18 @@ class FairnessMetrics:
                         self.result[i]["fair_metric_values"][j] = (None, None, None)
                     else:
                         self.result[i]["fair_metric_values"][j] = self.result[i]['fair_metric_values'][j][-1] + (2*np.std([a_tuple[0] for a_tuple in self.result[i]["fair_metric_values"][j]]),)
+        eval_pbar.update(6)
 
-    def _execute_all_fair_map(metric_obj, index):
+    def _execute_all_fair_map(metric_obj, index, eval_pbar, worker_progress):
         """
         Maps each thread's work for execute_all_fair()
+        Parameters
+        ----------
+        metric_obj :
+        
+        index : 
         """
+        prog = round(worker_progress/(len(index)), 2) if (len(index)) != 0 else 0
         for idx in index:
             metric_obj.y_true = [model.y_true[idx] for model in metric_obj.use_case_object.model_params]
             metric_obj.y_prob = [model.y_prob[idx] if model.y_prob is not None else None for model in metric_obj.use_case_object.model_params] 
@@ -242,12 +259,13 @@ class FairnessMetrics:
                 metric_obj.curr_p_var = i
                 metric_obj.tp_p, metric_obj.fp_p, metric_obj.tn_p, metric_obj.fn_p, metric_obj.tp_u, metric_obj.fp_u, metric_obj.tn_u, metric_obj.fn_u  = metric_obj.use_case_object._get_confusion_matrix(metric_obj.y_true[0], metric_obj.y_pred[0], metric_obj.sample_weight[0], i, metric_obj.feature_mask)            
                 for j in metric_obj._use_case_metrics['fair']:
-                            metric_obj.result[i]["fair_metric_values"][j].append(metric_obj.map_fair_metric_to_method[j]())
+                            metric_obj.result[i]["fair_metric_values"][j].append(metric_obj.map_fair_metric_to_method[j](obj=metric_obj))
 
+            eval_pbar.update(prog)
         return metric_obj.result
 
 
-    def translate_metric(self,metric_name, **kwargs ):
+    def translate_metric(self, metric_name, **kwargs ):
 
         """
         Computes the primary fairness metric value and its associate value for the privileged group, for the feature importance section.
@@ -577,10 +595,16 @@ class FairnessMetrics:
 
         if 'y_pred_new' in kwargs:
             y_pred=kwargs['y_pred_new'][0]
-
-        rmse_p = mean_squared_error(np.array(y_true)[mask], np.array(y_pred)[mask], np.array(self.sample_weight[0])[mask]) ** 0.5
-        rmse_u = mean_squared_error(np.array(y_true)[~mask], np.array(y_pred)[~mask], np.array(self.sample_weight[0])[~mask]) ** 0.5
-
+        
+        if self.sample_weight[0] is not None: 
+            sample_weight_p = np.array(self.sample_weight[0])[mask]
+            sample_weight_u = np.array(self.sample_weight[0])[~mask]
+        else:
+            sample_weight_p = None
+            sample_weight_u = None
+        
+        rmse_p = mean_squared_error(y_true=np.array(y_true)[mask], y_pred=np.array(y_pred)[mask], sample_weight=sample_weight_p) ** 0.5
+        rmse_u = mean_squared_error(y_true=np.array(y_true)[~mask], y_pred=np.array(y_pred)[~mask], sample_weight=sample_weight_u) ** 0.5
         return (rmse_p - rmse_u, rmse_p)
 
     def _compute_mape_parity(self, **kwargs):
@@ -603,9 +627,16 @@ class FairnessMetrics:
 
         if 'y_pred_new' in kwargs:
             y_pred=kwargs['y_pred_new'][0]
+            
+        if self.sample_weight[0] is not None: 
+            sample_weight_p = np.array(self.sample_weight[0])[mask]
+            sample_weight_u = np.array(self.sample_weight[0])[~mask]
+        else:
+            sample_weight_p = None
+            sample_weight_u = None
 
-        mape_p = mean_absolute_percentage_error(np.array(y_true)[mask], np.array(y_pred)[mask], np.array(self.sample_weight[0])[mask])
-        mape_u = mean_absolute_percentage_error(np.array(y_true)[~mask], np.array(y_pred)[~mask], np.array(self.sample_weight[0])[~mask])
+        mape_p = mean_absolute_percentage_error(y_true=np.array(y_true)[mask], y_pred=np.array(y_pred)[mask], sample_weight=sample_weight_p)
+        mape_u = mean_absolute_percentage_error(y_true=np.array(y_true)[~mask], y_pred=np.array(y_pred)[~mask], sample_weight=sample_weight_u)
 
         return (mape_p - mape_u, mape_p)
 
@@ -659,8 +690,16 @@ class FairnessMetrics:
         elif 'y_pred_new' in kwargs:
             y_prob=kwargs['y_pred_new'][0]
 
-        log_loss_p = log_loss(np.array(y_true)[mask], np.array(y_prob)[mask])
-        log_loss_u = log_loss(np.array(y_true)[~mask], np.array(y_prob)[~mask])
+        if self.sample_weight[0] is not None: 
+            sample_weight_p = np.array(self.sample_weight[0])[mask]
+            sample_weight_u = np.array(self.sample_weight[0])[~mask]
+
+        else:
+            sample_weight_p = None
+            sample_weight_u = None        
+
+        log_loss_p = log_loss(y_true=np.array(y_true)[mask], y_pred=np.array(y_prob)[mask], sample_weight = sample_weight_p)
+        log_loss_u = log_loss(y_true=np.array(y_true)[~mask], y_pred=np.array(y_prob)[~mask], sample_weight = sample_weight_u)
 
         return (log_loss_p - log_loss_u, log_loss_p)
 
@@ -684,15 +723,21 @@ class FairnessMetrics:
         
         if y_prob is None:
             return (None, None)
-        
             
         if 'y_pred_new' in kwargs:
             y_prob=kwargs['y_pred_new'][0]
-       
 
-        roc_auc_score_p = roc_auc_score(np.array(y_true)[mask], np.array(y_prob)[mask])
-        roc_auc_score_u = roc_auc_score(np.array(y_true)[~mask], np.array(y_prob)[~mask])
-        
+        if self.sample_weight[0] is not None: 
+            sample_weight_p = np.array(self.sample_weight[0])[mask]
+            sample_weight_u = np.array(self.sample_weight[0])[~mask]
+
+        else:
+            sample_weight_p = None
+            sample_weight_u = None
+
+        roc_auc_score_p = roc_auc_score(y_true=np.array(y_true)[mask], y_score=np.array(y_prob)[mask], sample_weight=sample_weight_p)
+        roc_auc_score_u = roc_auc_score(y_true=np.array(y_true)[~mask], y_score=np.array(y_prob)[~mask], sample_weight=sample_weight_u)
+            
         return (roc_auc_score_p - roc_auc_score_u, roc_auc_score_p)
 
     def _compute_equal_opportunity(self, **kwargs):
