@@ -8,12 +8,13 @@ from .fairness.credit_scoring import CreditScoring
 from .fairness.customer_marketing import CustomerMarketing
 from .ErrorCatcher import VeritasError
 import sys
+from collections.abc import Iterable
 
 class ModelContainer(object):
     """Helper class that holds the model parameters required for computations in all use cases."""
 
-    def __init__(self, y_true, p_var, p_grp, x_train,  x_test, model_object, model_type, model_name = 'auto', y_pred=None, y_prob=None, y_train=None, protected_features_cols=None, train_op_name="fit",
-                 predict_op_name ="predict", feature_imp=None, sample_weight=None, pos_label=[[1]], neg_label=None):
+    def __init__(self, y_true, p_var, p_grp, model_type, model_name = 'auto', y_pred=None, y_prob=None, y_train=None, protected_features_cols=None, train_op_name="fit",
+                 predict_op_name ="predict", feature_imp=None, sample_weight=None, sample_weight_train=None, pos_label=[[1]], neg_label=None, x_train=None, x_test=None, model_object=None):
         """
         Parameters
         ----------
@@ -117,7 +118,8 @@ class ModelContainer(object):
         self.predict_op_name = predict_op_name
         self.feature_imp = feature_imp
         self.sample_weight = sample_weight
-        self.model_name = model_name[0:20]
+        self.sample_weight_train = sample_weight_train        
+        self.model_name = model_name
         self.model_type = model_type
         if model_name == 'auto':
             self.model_name = model_type   
@@ -130,11 +132,28 @@ class ModelContainer(object):
         ## Define the ranges 
 
         
-        #check_y_label = None
+        check_y_label = None
 
-        check_y_label = list(set(y_true))
-        check_p_grp ={}
+        check_p_grp = None
+
+        self.check_protected_columns()
+
+        self.check_pos_label()
+
+        if y_true is not None :
+            check_y_label = list(set(y_true))
+        if p_var is not None :
+            check_p_grp=dict.fromkeys(p_var)
+
+
+        # Dictionary for input data
+        if protected_features_cols is None and type(self.x_test) == pd.DataFrame:
+            self.protected_features_cols = self.x_test[p_var]
+        for var in check_p_grp.keys():
+            check_p_grp[var] = self.protected_features_cols[var].unique()
+            
         NoneType = type(None)
+
         check_model_type = []
         for usecase in Fairness.__subclasses__():
         #     print(usecase)
@@ -146,21 +165,22 @@ class ModelContainer(object):
         # if value range is a tuple, will be condsidered as a numerical range
         # if value range is a list/set of str, will be considered as a collection of available values
         self._input_validation_lookup = {
-        "y_true": [(list, np.ndarray, pd.Series), None],
+        "y_true": [(list, np.ndarray, pd.Series), ],
         "y_train": [(NoneType, list, np.ndarray, pd.Series), check_y_label], ## y_pred unique values should be subset of y_true unique values
         "y_pred": [(NoneType, list, np.ndarray, pd.Series), check_y_label], ## y_pred unique values should be subset of y_true unique values
-        "y_prob": [(NoneType, list, np.ndarray, pd.Series), (-0.01,1.01)],
+        "y_prob": [(NoneType, list, np.ndarray, pd.Series, pd.DataFrame), (-0.01,1.01)],
         "p_var":  [(list,), str],
         "protected_features_cols": [(NoneType, pd.DataFrame), self.p_var],
         "p_grp":  [(dict,), check_p_grp], ## do a subset check. for each key in p_grp should be inside check_p_grp
-        "x_train": [(pd.DataFrame, str), None],
-        "x_test":  [(pd.DataFrame, str), None],
+        "x_train": [(NoneType, pd.DataFrame, str), None],
+        "x_test":  [(NoneType, pd.DataFrame, str), None],
         "train_op_name": [(str,), None],
         "predict_op_name": [(str,), None],
         "feature_imp":     [(NoneType, pd.DataFrame), (np.dtype('O'), np.dtype('float64'))], ### use pandas.Dataframe.dtypes to check if the 1st columns is string (object) and 2nd column is numeric (float64)
         "sample_weight":   [(NoneType, list, np.ndarray), (0, np.inf)], ## should be > 0
+        "sample_weight_train":   [(NoneType, list, np.ndarray), (0, np.inf)], ## should be > 0
         "model_name":      [(str,), None],
-         "model_type":      [(str,), check_model_type],
+        "model_type":      [(str,), check_model_type],
         "pos_label":       [(list,), check_y_label], ## should be subset of y_true unique values
         "neg_label":       [(NoneType, list), check_y_label] ## should be subset of y_true unique values
         }
@@ -170,14 +190,7 @@ class ModelContainer(object):
         # #print(message)
         # assert status, message
         check_datatype(self)
-        
-
-        # Dictionary for input data
-        if protected_features_cols is None and type(self.x_test) == pd.DataFrame:
-            self.protected_features_cols = self.x_test[p_var]
-        for var in self.p_grp.keys():
-            check_p_grp[var] = self.protected_features_cols[var].unique()
-
+        self.model_name = model_name[0:20]
 
         #converts to np array
         self.y_true = np.array(self.y_true)
@@ -189,7 +202,8 @@ class ModelContainer(object):
             self.y_train = np.array(self.y_train)
         if self.sample_weight is not None :
             self.sample_weight = np.array(self.sample_weight)
-
+        if self.sample_weight_train is not None :
+            self.sample_weight_train = np.array(self.sample_weight_train)
         # status, message = self.check_data_consistency()
         # #print(status)
         # #print(message)
@@ -212,6 +226,7 @@ class ModelContainer(object):
         # assert status, message
 
         self.check_data_consistency()
+        self.check_label_consistency()
 
         # status, message = self.check_data_consistency()
         # #print(status)
@@ -219,17 +234,52 @@ class ModelContainer(object):
         # assert status, message
 
         #if user provide neg_label, need to 
-        #label encoding to transfrom y_ture
-        if len(self.y_true.shape) == 1 and self.y_true.dtype.kind in ['i','O','S']:
-            self.y_true, self.pos_label2 = check_label(self.y_true, self.pos_label, self.neg_label)
-        if self.y_pred is not None and len(self.y_pred.shape) == 1 and self.y_pred.dtype.kind in ['i','O','S']:
+        #label encoding to transfrom y_true
+        if len(self.y_true.shape) == 1 and self.y_true.dtype.kind in ['i','O','U']:
+            self.y_true, self.pos_label2 = check_label(self.y_true, self.pos_label, self.neg_label)  
+        if self.y_pred is not None and len(self.y_pred.shape) == 1 and self.y_pred.dtype.kind in ['i','O','U']:
             self.y_pred, self.pos_label2 = check_label(self.y_pred, self.pos_label, self.neg_label)
+            
+        if self.y_train is not None and len(self.y_train.shape) == 1 and self.y_train.dtype.kind in ['i','O','U']:
+            self.y_train, self.pos_label2 = check_label(self.y_train, self.pos_label, self.neg_label)
+ 
+            
 
         # if err_list != []:
         #     self.err.push(err_list[0][0], var_name_a=err_list[0][1], some_string=err_list[0][2], value=err_list[0][3])
         # else:
         #     print(err_list)
         # self.err.pop()
+        
+    def check_protected_columns(self):     
+        # Check p_var x_test/protected_feature_columns consistency
+        err_ = []
+        successMsg = "data consistency check completed without issue"
+
+        if type(self.p_var) != list:
+            err_.append(['type_error', "p_var", type(self.p_var), "list"])
+
+        elif self.protected_features_cols is None and self.x_test is None :
+            err_.append(['length_error', "protected_features_cols and x_test", "None for both", "not both are None"])
+        
+        
+        elif self.protected_features_cols is not None :
+            if sum([x in self.p_var for x in self.protected_features_cols.columns]) != len(self.p_var): 
+                err_.append(['value_error', "p_var", str(self.p_var), str(list(self.protected_features_cols.columns))])
+        elif self.protected_features_cols is None and self.x_test is not None :           
+            if sum([x in self.p_var for x in self.x_test.columns]) != len(self.p_var) : 
+                err_.append(['value_error', "p_var", str(self.p_var), str(list(self.x_test.columns))])
+          
+        if err_ == []:
+            err_ = successMsg
+            return err_
+        else:
+            for i in range(len(err_)):
+                self.err.push(err_[i][0], var_name=err_[i][1], given=err_[i][2], expected=err_[i][3],
+                            function_name="check_protected_columns")
+            self.err.pop()
+            
+
 
     def check_data_consistency(self):
         """
@@ -247,22 +297,14 @@ class ModelContainer(object):
         # errMsg = "data consistency error"
         successMsg = "data consistency check completed without issue"
         # errMsgFormat = "\n    {}: given {} size {}, expected {}"
+        # for usecase in Fairness.__subclasses__():
+        #     model_type_to_metric_lookup = (getattr(usecase, "_model_type_to_metric_lookup"))
+        #     if self.model_type in model_type_to_metric_lookup.keys():
+        #         if len(set(self.y_true)) != (model_type_to_metric_lookup.get(self.model_type)[1]) :
+        #             err_.append(['length_error', "y_true labels", str(len(set(self.y_true))), str(model_type_to_metric_lookup.get(self.model_type)[1])])
 
         test_row_count = self.y_true.shape[0] #from y_true (ground truth for test dataset)pred on x_test, result in y_pred compare with y_true
 
-        # feature_imp rowcount <= 20
-        if self.feature_imp is not None:
-            feature_imp_cols = len(self.feature_imp.columns)
-            feature_imp_rows = len(self.feature_imp.index)
-            if feature_imp_cols != 2:
-                # errMsg += errMsgFormat.format("feature_imp", "column", str(feature_imp_cols), "2")
-                err_.append(['length_error', "feature_imp column", str(feature_imp_cols), "2"])
-                # check_status = check_status and False
-            if feature_imp_rows > 20:
-                # errMsg += errMsgFormat.format("feature_imp", "row", str(feature_imp_rows), "<=20")
-                err_.append(['length_error', "feature_imp row", str(feature_imp_rows), "<=20"])
-                # check_status = check_status and False
-
         # check protected_features_cols
         # check cols of protected_features_cols is same as p_var
         pro_f_cols_rows = len(self.protected_features_cols.index)
@@ -296,6 +338,11 @@ class ModelContainer(object):
                     err_.append(['length_error', "x_train row", str(x_train_rows), str(train_row_count)])
                     # errMsg += errMsgFormat.format("x_train","row", str(x_train_rows), str(train_row_count))
                     # check_status = check_status and False
+            if self.sample_weight is not None:
+                if self.sample_weight_train is None :
+                    err_.append(['length_error', "sample_weight_train", "None", "not None"])                    
+                elif len(self.sample_weight_train) != train_row_count:
+                    err_.append(['length_error', "sample_weight_train row", str(len(self.sample_weight_train) ), str(train_row_count)])
 
 
         #y_prob should be float
@@ -313,33 +360,29 @@ class ModelContainer(object):
 
         #check pos_label size and neg_label size
 
-        try:
-            for usecase in Fairness.__subclasses__():
-                model_type_to_metric_lookup = (getattr(usecase, "_model_type_to_metric_lookup"))
-                if self.model_type in model_type_to_metric_lookup.keys():
-                        #assumption: model_type is unique across all use cases
-                    label_size = (model_type_to_metric_lookup.get(self.model_type)[1])/2
-                    break
-        except:
-            pass
+
+        for usecase in Fairness.__subclasses__():
+            model_type_to_metric_lookup = (getattr(usecase, "_model_type_to_metric_lookup"))
+            if self.model_type in model_type_to_metric_lookup.keys():
+                    #assumption: model_type is unique across all use cases
+                label_size = int((model_type_to_metric_lookup.get(self.model_type)[1])/2)
+                
 
         # if label size requirement is -1/2, no need to check
 
-        try:
-            if label_size > 0:
-                if self.neg_label is not None:
-                    neg_label_size = len(self.neg_label)
-                    if neg_label_size != label_size:
-                        # errMsg += errMsgFormat.format("neg_label", "", str(neg_label_size), str(label_size))
-                        err_.append(['length_error', "x_train column", str(x_train_cols), str(x_test_cols)])
-                        # check_status = check_status and False
-                    pos_label_size = len(self.pos_label)
-                    if pos_label_size != label_size:
-                        # errMsg += errMsgFormat.format("pos_label", "", str(pos_label_size), str(label_size))
-                        err_.append(['length_error', "pos_label", str(pos_label_size), str(label_size)])
-                        # check_status = check_status and False
-        except:
-            pass
+        if label_size > 0:
+            if self.neg_label is not None:
+                neg_label_size = len(self.neg_label)
+                if neg_label_size != label_size:
+                    # errMsg += errMsgFormat.format("neg_label", "", str(neg_label_size), str(label_size))
+                    err_.append(['length_error', "neg_label", str(neg_label_size), str(label_size)])
+                    # check_status = check_status and False
+            pos_label_size = len(self.pos_label)
+            if pos_label_size != label_size:
+                # errMsg += errMsgFormat.format("pos_label", "", str(pos_label_size), str(label_size))
+                err_.append(['length_error', "pos_label", str(pos_label_size), str(label_size)])
+                # check_status = check_status and False
+
 
         #y_pred and y_prob should not be both none
         if self.y_pred is None and self.y_prob is None:
@@ -388,14 +431,52 @@ class ModelContainer(object):
             err_ = successMsg
             return err_
         else:
-            self.err.push(err_[0][0], var_name=err_[0][1], given=err_[0][2], expected=err_[0][3],
-                          function_name="check_data_consistency")
+            for i in range(len(err_)):
+                self.err.push(err_[i][0], var_name=err_[i][1], given=err_[i][2], expected=err_[i][3],
+                            function_name="check_data_consistency")
             self.err.pop()
 
 
         # msg = successMsg if check_status else errMsg
         # return check_status, msg
-    
+
+    def check_label_consistency(self):
+        err_ = []
+        # check_status = True
+        # errMsg = "data consistency error"
+        successMsg = "data consistency check completed without issue"
+        if self.y_pred is not None :
+            if set(self.y_true) != set(self.y_pred):
+                err_.append(['value_error', "y_pred labels", str(set(self.y_pred)), str(set(self.y_true))])
+        
+        if self.y_train is not None :
+            if set(self.y_true) != set(self.y_train):
+                err_.append(['value_error', "y_train labels", str(set(self.y_train)), str(set(self.y_true))])                
+
+        if err_ == []:
+            err_ = successMsg
+            return err_
+        else:
+            for i in range(len(err_)):
+                self.err.push(err_[i][0], var_name=err_[i][1], given=err_[i][2], expected=err_[i][3],
+                            function_name="check_label_consistency")            
+            self.err.pop()
+
+    def check_pos_label(self):
+        def flatten(l):
+            for el in l:
+                if isinstance(el, Iterable) and not isinstance(el, (str, bytes)):
+                    yield from flatten(el)
+                else:
+                    yield el
+                    
+        for i in self.pos_label:
+            if type(i) == list:
+                lst = list(flatten(i))
+                if len(lst) > 1:
+                    self.err.push("length_error", var_name="pos_label nested list " + str(i), given=str(len(lst)), expected='1', function_name="check_pos_label")
+        self.err.pop()
+
     def clone(self,  y_true, model_object, y_pred=None, y_prob=None, y_train=None, train_op_name="fit",
                  predict_op_name ="predict", feature_imp=None, sample_weight=None, pos_label=[[1]], neg_label=None):
 
@@ -451,7 +532,7 @@ class ModelContainer(object):
         """
         
         clone_obj = ModelContainer(y_true = y_true, p_var = self.p_var, p_grp = self.p_grp, x_train= self.x_train,  x_test = self.x_test, model_object = model_object, model_type = self.model_type, model_name = "clone", y_pred=y_pred, y_prob=y_prob, y_train = y_train, protected_features_cols=self.protected_features_cols, train_op_name=train_op_name,
-                 predict_op_name = predict_op_name, feature_imp=self.feature_imp, sample_weight=self.sample_weight, pos_label=pos_label, neg_label=neg_label)
+                 predict_op_name = predict_op_name, feature_imp=self.feature_imp, sample_weight=self.sample_weight, sample_weight_train=self.sample_weight_train, pos_label=pos_label, neg_label=neg_label)
         
         return clone_obj
 
