@@ -7,6 +7,7 @@ import multiprocessing
 import math
 from .config.constants import Constants
 from .ErrorCatcher import VeritasError
+from pathlib import Path
 
 def check_datatype(obj_in):
     """
@@ -61,21 +62,31 @@ def check_datatype(obj_in):
         if cur_type not in exp_type:
             # errMsg += errMsgFormat.format(str(var_name),str(cur_type), str(exp_type))
             err_.append(['type_error', str(var_name), str(cur_type), str(exp_type)])
+
+        if var_name == "p_grp":
+            for i in getattr(obj_in, var_name).values():
+                if type(i) != list:
+                    err_.append(['type_error', "p_grp values", str(type(i)), "list"])
+
             # check_status = check_status and False
-        #to check callable functions
-        if var_name in ["train_op_name", "predict_op_name"]:
-            try:
-                callable(getattr(obj_in.model_object, var))
-            except AttributeError:
-                # errMsg += errMsgFormat.format("'"+var_name+"'"," non-callable " + "'"+var_name+"'", "callable"+str(var_name)) #check this line
-                err_.append(['type_error', var_name, " non-callable ", "callable"])
-                # check_status = check_status and False
+        # #to check callable functions
+        # if var_name in ["train_op_name", "predict_op_name"]:
+        #     if obj_in.model_object is not None :
+        #         try:
+        #             callable(getattr(obj_in.model_object, var))
+        #         except AttributeError:
+        #             # errMsg += errMsgFormat.format("'"+var_name+"'"," non-callable " + "'"+var_name+"'", "callable"+str(var_name)) #check this line
+        #             err_.append(['type_error', var_name, " non-callable ", "callable"])
+        #             # check_status = check_status and False
+        
+
 
     if err_ == []:
         err_ = successMsg
         return err_
     else:
-        err.push(err_[0][0], var_name=err_[0][1], given=err_[0][2], expected=err_[0][3], function_name="check_datatype")
+        for i in range(len(err_)):
+            err.push(err_[i][0], var_name=err_[i][1], given=err_[i][2], expected=err_[i][3], function_name="check_datatype")
         err.pop()
 
     # return err_
@@ -181,11 +192,14 @@ def check_value(obj_in):
         # eg check feature_imp
         elif var_value_type == pd.DataFrame and var_range_type in numeric_range_types:
             var_value_types = var_value.dtypes
-            for i, tp in zip(range(len(var_value_types)), var_value_types):
-                if tp != var_range[i]:
-                    # errMsg += errMsgFormat.format(var_name + "column " +str(i), str(tp), str(var_range[i]))
-                    err_.append(['column_value_error', var_name, str(var_range[i])], str(i))
-                    # check_status = check_status and False
+            if len(var_value_types) != len(var_range):
+                err_.append(['length_error', var_name, len(var_value_types), len(var_range)])
+            else:
+                for i, tp in zip(range(len(var_value_types)), var_value_types):
+                    if tp != var_range[i]:
+                        # errMsg += errMsgFormat.format(var_name + "column " +str(i), str(tp), str(var_range[i]))
+                        err_.append(['column_value_error', var_name, str(var_range[i])], str(i))
+                        # check_status = check_status and False
         # eg check p_grp
         elif var_value_type == dict and var_range_type == dict:
             keyset1 = set(var_value.keys())
@@ -228,7 +242,8 @@ def check_value(obj_in):
         err_ = successMsg
         return err_
     else:
-        err.push(err_[0][0], var_name=err_[0][1], given=err_[0][2], expected=err_[0][3], function_name="check_value")
+        for i in range(len(err_)):
+            err.push(err_[i][0], var_name=err_[i][1], given=err_[i][2], expected=err_[i][3], function_name="check_value")
         err.pop()
 
     # return err_
@@ -256,55 +271,55 @@ def convert_to_set(var):
         result = var
     return result
 
-def bootstrap_conf_int(metric_obj, metric_name, k=50):
-    """
-    Calculates the confidence interval for any given metric
+# def bootstrap_conf_int(metric_obj, metric_name, k=50):
+#     """
+#     Calculates the confidence interval for any given metric
 
-    Parameters
-    ----------------
-    metric_obj : object
-            A single initialized Fairness use case object (CreditScoring, CustomerMarketing, etc.)
+#     Parameters
+#     ----------------
+#     metric_obj : object
+#             A single initialized Fairness use case object (CreditScoring, CustomerMarketing, etc.)
 
-    metric_name : str
-            The performance or fairness metric name.
+#     metric_name : str
+#             The performance or fairness metric name.
 
-    k : int, default=50
-            The number of data points computed to obtain the confidence interval
+#     k : int, default=50
+#             The number of data points computed to obtain the confidence interval
 
-    Returns:
-    ---------------
-    np.mean(out), 2*np.std(out) : tuple of floats
-            Mean and confidence interval values
-    """
-    n = len(metric_obj.y_true[0])
-    out = []
-    np.random.seed(123)
-    for i in range(k):
-        idx = np.random.choice(n, n, replace=True)
-        out.append(metric_obj.translate_metric(metric_name, idx=idx)[0])
-    out = np.array(out)
-    return np.mean(out), 2*np.std(out)
+#     Returns:
+#     ---------------
+#     np.mean(out), 2*np.std(out) : tuple of floats
+#             Mean and confidence interval values
+#     """
+#     n = len(metric_obj.y_true[0])
+#     out = []
+#     np.random.seed(123)
+#     for i in range(k):
+#         idx = np.random.choice(n, n, replace=True)
+#         out.append(metric_obj.translate_metric(metric_name, idx=idx)[0])
+#     out = np.array(out)
+#     return np.mean(out), 2*np.std(out)
 
     
     
-def format_uncertainty(mean_val, conf_int):
-    """
-    Formats uncertainty
+# def format_uncertainty(mean_val, conf_int):
+#     """
+#     Formats uncertainty
 
-    Parameters
-    ------------
-    mean_val: float
-            Mean value
+#     Parameters
+#     ------------
+#     mean_val: float
+#             Mean value
 
-    conf_int: float
-            Confidence interval value
+#     conf_int: float
+#             Confidence interval value
 
-    Returns
-    ------------
-    Mean and confidence interval in standardised number of decimal places
-    """
-    # return f"{mean_val:.3f} +/- {conf_int:.3f}"
-    return "{:.{decimal_pts}} +/- {:.{decimal_pts}}".format(mean_val, conf_int, decimal_pts=Constants().decimals)
+#     Returns
+#     ------------
+#     Mean and confidence interval in standardised number of decimal places
+#     """
+#     # return f"{mean_val:.3f} +/- {conf_int:.3f}"
+#     return "{:.{decimal_pts}} +/- {:.{decimal_pts}}".format(mean_val, conf_int, decimal_pts=Constants().decimals)
 
 
 def check_label(y, pos_label, neg_label=None):
@@ -343,38 +358,64 @@ def check_label(y, pos_label, neg_label=None):
     ## 2, 3 => treatment (others, rejected/responded)
     err = VeritasError()
     err_= []
+
     if neg_label is not None and len(neg_label) == 2:
         y_bin = y
         n=0
+
         row = y_bin == pos_label[0]  
-        y_bin[row] = 'TR' 
+        # y_bin[row] = 'TR' 
+        indices_pos_0 = [i for i, x in enumerate(y_bin) if x == pos_label[0][0]]
         n += np.sum(row)
+
         row = y_bin == pos_label[1]  
-        y_bin[row] = 'CR' 
+        # y_bin[row] = 'CR' 
+        indices_pos_1 = [i for i, x in enumerate(y_bin) if x == pos_label[1][0]]
         n += np.sum(row)
+
         row = y_bin == neg_label[0]  
-        y_bin[row] = 'TN' 
+        # y_bin[row] = 'TN' 
+        indices_neg_0 = [i for i, x in enumerate(y_bin) if x == neg_label[0][0]]
         n += np.sum(row)
+
         row = y_bin == neg_label[1]  
-        y_bin[row] = 'CN' 
-        n += np.sum(row)        
+        # y_bin[row] = 'CN' 
+        indices_neg_1 = [i for i, x in enumerate(y_bin) if x == neg_label[1][0]]
+        n += np.sum(row)     
+
+        for i in indices_pos_0:
+            y_bin[i] = "TR"
+        for i in indices_pos_1:
+            y_bin[i] = "CR"
+        for i in indices_neg_0:
+            y_bin[i] = "TN"
+        for i in indices_neg_1:
+            y_bin[i] = "CN"
+
         if n != len(y_bin):
             # raise ValueError('y dataset contains labels other than provided. Please provide valid pos_label and neg_label.')
             err_.append(['conflict_error', "y dataset", "inconsistent values", pos_label + neg_label])
-            err.push(err_[0][0], var_name_a=err_[0][1], some_string=err_[0][2], value=err_[0][3],
-                     function_name="check_label")
-            err.pop()
+            for i in range(len(err_)):
+                err.push(err_[i][0], var_name_a=err_[i][1], some_string=err_[i][2], value=err_[i][3],
+                        function_name="check_label")
         pos_label2 = [['TR'],['CR']]
     else:
         y_bin = y
-        row = y_bin == pos_label[0]  
+        row = np.isin(y_bin, pos_label[0])
+        if sum(row) == len(y_bin) :
+            err_.append(['length_error', "number of labels", "1", "2"])
+            for i in range(len(err_)):
+                err.push(err_[i][0], var_name=err_[i][1], given=err_[i][2], expected=err_[i][3],
+                        function_name="check_label")
+
         y_bin[row] = 1 
         y_bin[~row] = 0
         pos_label2 = [[1]]
     
-    if y_bin.dtype.kind == 'i':
+    if y_bin.dtype.kind in ['i']:
         y_bin  = y_bin.astype(np.int8)
 
+    err.pop()
 
     return y_bin, pos_label2
 
@@ -421,8 +462,12 @@ def check_multiprocessing(n_threads):
     return n_threads
 
 def test_function_cs():
+    from .fairness import CreditScoring
+    from . import ModelContainer
+    import pickle
     #Load Credit Scoring Test Data
-    file = r".\veritas\resources\data\credit_score_dict.pickle"
+    PATH = Path(__file__).parent.joinpath('resources', 'data')
+    file = PATH/"credit_score_dict.pickle"
     input_file = open(file, "rb")
     cs = pickle.load(input_file)
 
@@ -437,7 +482,7 @@ def test_function_cs():
     p_grp = {'SEX': [1], 'MARRIAGE':[1]}
     x_train = cs["X_train"]
     x_test = cs["X_test"]
-    model_object = LogisticRegression(C=0.1)
+    model_object = cs["model"]
     model_name = "credit scoring"
     model_type = "default"
     y_prob = cs["y_prob"]
@@ -445,14 +490,13 @@ def test_function_cs():
     #rejection inference
     num_applicants = {'SEX': [3500, 5000], 'MARRIAGE':[3500, 5000]}
     base_default_rate = {'SEX': [0.10,0.05], 'MARRIAGE':[0.10,0.05]}
-    model_object = LRwrapper(model_object)
     
     container = ModelContainer(y_true = y_true, y_train = y_train, p_var = p_var, p_grp = p_grp, x_train = x_train,  x_test = x_test, model_object = model_object, model_type  = model_type,model_name =  model_name, y_pred= y_pred, y_prob= y_prob)
     cre_sco_obj= CreditScoring(model_params = [container], fair_threshold = 0.43, fair_concern = "eligible", fair_priority = "benefit", fair_impact = "significant", perf_metric_name = "balanced_acc", fair_metric_name = "equal_opportunity")
     cre_sco_obj.k = 1
     cre_sco_obj.evaluate(output = False)
     result = cre_sco_obj.perf_metric_obj.result, cre_sco_obj.fair_metric_obj.result
-    file = r".\veritas\resources\data\credit_score_sample.pickle"
+    file = PATH/"credit_score_sample.pickle"
     input_file = open(file, "rb")
     sample = pickle.load(input_file)
     if result[0]['perf_metric_values'] == sample[0]['perf_metric_values'] and result[1] == sample[1]:
@@ -461,9 +505,13 @@ def test_function_cs():
         print('The test results are abnormal')
         
 def test_function_cm():
-#Load Phase 1-Customer Marketing Uplift Model Data, Results and Related Functions
-    file_prop = r".\veritas\resources\data\mktg_uplift_acq_dict.pickle"
-    file_rej = r".\veritas\resources\data\mktg_uplift_rej_dict.pickle"
+    from .fairness import CustomerMarketing
+    from . import ModelContainer
+    import pickle
+    #Load Phase 1-Customer Marketing Uplift Model Data, Results and Related Functions
+    PATH = Path(__file__).parent.joinpath('resources', 'data')
+    file_prop = PATH/"mktg_uplift_acq_dict.pickle"
+    file_rej = PATH/"mktg_uplift_rej_dict.pickle"
     input_prop = open(file_prop, "rb")
     input_rej = open(file_rej, "rb")
     cm_prop = pickle.load(input_prop)
@@ -517,10 +565,10 @@ def test_function_cm():
     cm_uplift_obj.k = 1
     cm_uplift_obj.evaluate(output = False)
     result = cm_uplift_obj.perf_metric_obj.result, cm_uplift_obj.fair_metric_obj.result
-    file = r".\veritas\resources\data\cm_uplift_sample.pickle"
+    file = PATH/"cm_uplift_sample.pickle"
     input_file = open(file, "rb")
     sample = pickle.load(input_file)
     if result[0]['perf_metric_values'] == sample[0]['perf_metric_values'] and result[1] == sample[1]:
         print("Evaluation of customer marketing performed normally")
     else:
-        print('The test results are abnormal')
+        print('The test results are abnormal')
\ No newline at end of file
